using GraphicsManager.Enums;
using GraphicsManager.Interfaces;
using GraphicsManager.Objects;
using GraphicsManager.Objects.Core;
using Luski.net.Structures.Public;
using OpenTK.Graphics.OpenGL4;
using OpenTK.Mathematics;
using OpenTK.Windowing.Common.Input;

namespace Luski.GUI.MainScreen.UI.PublicServers;

public class Channel : UserControl
{
    //private Rectangle SelectedRct, SelectedRctL, SelectedRctR;
    private ChannelSelector CS;
    //public readonly static Texture[] SelectedTextures = new Texture[] {null!, null!, null!};

    public static Texture? seltec = null;

    public SocketChannel CurrentChannel { get; set; }

    private Channel(Stream UserIcon, ChannelSelector cs, SocketChannel chan)
    :base(seltec)
    {
        CS = cs;
        CurrentChannel = chan;
        Size = new((int)(307 * Globals.Settings.Scale), (int)(40* Globals.Settings.Scale));

        TextureDisplay = TextureDisplay.HorizontalCenter;
        Shader = Rectangle.DefaultAlphaShader[Globals.ms.Context];
        int i = (int)(4 * Globals.Settings.Scale);
        r = new Rectangle(Globals.ms.TextureManager.GetAlphaCircle()) 
        {
            Location = new(i,i,0),
            Size = new ((int)(32 * Globals.Settings.Scale)),
            IgnoreHover = true
        };
        Texture tex;
        tex = Globals.ms.TextureManager.AddTexture(UserIcon);
        GC.Collect();
        UserIcon.Dispose();
        tex.Unit = TextureUnit.Texture1;
        r.Textures.Add(tex);
        r.Shader = Rectangle.DefaultAlphaTextureShader[Globals.ms.Context];
        Controls.Add(r);
        ChannelName = new Label(Globals.DefaultFont)
        {
            Text = chan.Name,
            Color = chan.Color.ToColor4(),
            IgnoreHover = true
        };
        Controls.Add(ChannelName);
        Clicked += AllOnClicked;
        ChannelName.Location = new((int)(40 * Globals.Settings.Scale),
            (Size.Y / 2) - ((int)ChannelName.Font.PixelHeight) + (ChannelName.PostiveTrueHeight / 2)
            , 0);
        ContextMenu = new((int)(150 * Globals.Settings.Scale));
        HoverMouse = MouseCursor.Hand;
    }

    public bool Selected { get; private set; }

    public async Task ToggleSelected()
    {
        try
        {
            Color4 bc = new(141,151,165,51);
            if (Selected)
            {
                bc = CS.BackgroundColor;
            }
            
            BlockDraw = true;
            Selected = !Selected;

            if (CS.Selected is not null && CS.Selected != this)
            {
                await CS.Selected.ToggleSelected();
            }
            BackgroundColor = bc;
            if (Selected)
            {
                
                CS.Selected = this;
                IReadOnlyList<SocketMessage> m;
                    m = await CurrentChannel.GetMessages(CancellationToken.None, 200);
                    //m = Array.Empty<SocketMessage>();
                Globals.ms.pc.ClearChat();
                await Globals.ms.pc.LoadChannel(CurrentChannel);
                _ = Globals.ms.pc.AddMessages(m);
                if (m.Count > 0)Globals.ms.pc.MessageFlow.ScrollToBottom();
            }
            BlockDraw = false;
            TryDraw();
        }
        catch (Exception e)
        {
            Console.WriteLine(e);
        }
    }
    public Label ChannelName;
    public Rectangle r;

    public static async Task<Channel> MakeChannel(SocketChannel chan, ChannelSelector cs)
    {
        Channel c = new(await chan.GetPicture(CancellationToken.None), cs, chan);
        return c;
    }

    private Task EditChannelButtonOnClicked(IRenderObject arg)
    {
        arg.ContextMenu!.HideContext(Window!);
        return Task.CompletedTask;
    }

    private Task ExportKeysButtonOnClicked(IRenderObject arg)
    {
        //_ = CurrentChannel.SendKeysToUsers(CancellationToken.None);
        ContextMenu!.HideContext(Window!);
        return Task.CompletedTask;
    }
    
    private async Task AllOnClicked(IRenderObject arg)
    {
        if (!Selected) await ToggleSelected();
        //if (ClickCon is not null) _ = ClickCon.Invoke(this);
    }

    //public event Func<Channel, Task>? ClickCon;
}