using System.Diagnostics;
using System.Reflection;
using GraphicsManager;
using GraphicsManager.Enums;
using GraphicsManager.Interfaces;
using GraphicsManager.Objects;
using GraphicsManager.Objects.Core;
using Luski.net.Interfaces;
using Luski.net.Structures.Main;
using Luski.net.Structures.Public;
using OpenTK.Mathematics;
using OpenTK.Windowing.Common.Input;
using Label = GraphicsManager.Objects.Label;

namespace Luski.GUI.MainScreen.UI.PublicServers;

public class ChatMessage : UserControl
{
    //readonly int padding = 10;
    private static Font TimeFont;// = Font.MakeFontFromSystem(13);
    private SocketMessage Msg { get; }
    private SocketChannel ch { get; }

    public PublicChat pc;

    private IRenderObject LastObject;
    private Label FirstL;

    private readonly double HorPadding = (12 * Globals.Settings.Scale),
        VerticalPadding = (12 * Globals.Settings.Scale);

    private static Dictionary<MainSocketRemoteUser, ContextMenu> Menues = new();
    private static Dictionary<MainSocketRemoteUser, List<ChatMessage>> Messages = new();

    public static async Task<ChatMessage> MakeChatMessage(PublicChat p, SocketMessage message)
    {
        IUser auth = await message.GetAuthor(CancellationToken.None);
        Role[] ra = await ((SocketUser)auth).GetRoles();
        Color c = ra[0].Color;
        Color4 c4 = new(c.R, c.G, c.B, c.A);
        return new ChatMessage(p, message, await message.GetParent(CancellationToken.None), auth, await auth.GetIcon(), c4);
    }


    private ChatMessage(PublicChat p, SocketMessage message, SocketChannel chan, IUser Author, Texture UserIcon, Color4 UserNameColor)
    {
        pc = p;
        Label label1;
        Size = new((int)(723.5 * Globals.Settings.Scale), (int)(37 * Globals.Settings.Scale));
        ch = chan;
        BackgroundColor = new(40, 40, 40, 255);
        Msg = message;
        Anchor = ObjectAnchor.Left | ObjectAnchor.Right;
        
        DateTime time = chan.Epoch.AddMilliseconds(Msg.ID >> 20).ToLocalTime();
        string timestr;
        if (time.Date == DateTime.Now.ToLocalTime().Date)
        {
            timestr = $"Today at {time.ToShortTimeString()}";
        }
        else if (time.Date == DateTime.Now.ToLocalTime().AddDays(-1).Date)
        {
            timestr = $"Yesterday at {time.ToShortTimeString()}";
        }
        else
        {
            timestr = $"{time:M/dd/yyyy h:mm tt}";
        }
       
        Rectangle r = new Rectangle(Globals.ms.TextureManager.AddTexture(Tools.GetResourceStream(Assembly.GetExecutingAssembly(),
            "Luski.Resources.Textures.Status.png"))) { Location = new((int)(10 * Globals.Settings.Scale), (int)(2 * Globals.Settings.Scale), 0), Size = new((int)(38 * Globals.Settings.Scale)) };
        //UserIcon.Unit = TextureUnit.Texture1;
        r.Shader = Rectangle.DefaultAlphaTextureShader[Globals.ms.Context];
        r.Textures.Add(UserIcon);
        Controls.Add(r);
        Controls.Add(label1 = new Label(Globals.DefaultFont) { Color = UserNameColor, Text = Author.DisplayName });
        label1.Location = new(
            (int)(54 * Globals.Settings.Scale),
            (int)(r.Location.Y + (r.Size.Y / 2) - (label1.Font.CurrentFont.Fonts[0].Size.Metrics.NominalHeight / 2) - label1.Size.Y + label1.Font.PixelHeight), 0);
        LastObject = label1;
        FirstL = label1;
        Controls.Add(new Label(Globals.TopTimeFont) { Location = new(label1.Location.X + label1.Size.X + (int)(8 * Globals.Settings.Scale), (int)(label1.Location.Y + label1.Font.PixelHeight - Globals.TopTimeFont.PixelHeight), 0), Text = timestr});
        if (!string.IsNullOrWhiteSpace(Msg.Context))
        {
            Label l;
            Controls.Add(l = new Label(Globals.MessageFont) { Location = new(LastObject.Location.X, (int)(LastObject.Location.Y + (LastObject as Label)!.Size.Y + VerticalPadding), 0), Text = message.Context});
            LastObject = l;
        }
        
        if (Msg.Files.Count > 0)
        {
            int row = 1;
            int filesonrow = 0;
            for (int i = 0; i < Msg.Files.Count; i++)
            {
                double lx = (HorPadding * filesonrow) + LastObject.Location.X + (333 * (filesonrow + 1));
                if (lx > Size.X)
                {
                    row++;
                    filesonrow = 0;
                    lx = (HorPadding * filesonrow) + LastObject.Location.X + (333 * (filesonrow + 1));
                }

                filesonrow++;
                IRenderObject cem = ContentEmbed.GetEmbed(this, Msg.Files[i], Msg.ChannelID).Result;
                cem.Location = new((int)(lx - 333), (int)(LastObject.Location.Y + 2 + LastObject.Size.Y + (HorPadding * row) + (66 * (row - 1))), 0);
                LastObject = cem;
                //lo = cem.Location;
               // si = cem.Size;
                Controls.Add(cem);
            }
        }

        if (LastObject is Label ll) Size = new(Size.X, (int)(ll.Location.Y + ll.Size.Y + VerticalPadding));
        else Size = new(Size.X ,(int)(LastObject.Location.Y + LastObject.Size.Y + VerticalPadding));
    }

    public async Task AddMessage(SocketMessage msg)
    {
        Label newLabel;
        if (!string.IsNullOrWhiteSpace(msg.Context))
        {
            newLabel = new(Globals.MessageFont)
            {
                Text = msg.Context,
                Tag = msg
            };
            if (LastObject is Label l)
            {
                newLabel.Location = new(FirstL.Location.X, (int)(l.Location.Y + l.Size.Y + VerticalPadding), 0);
            }
            else
            {
                newLabel.Location = new(FirstL.Location.X, Size.Y, 0);
            }
            bool result = Uri.TryCreate(newLabel.Text, UriKind.Absolute, out Uri? uriResult) 
                          && (uriResult.Scheme == Uri.UriSchemeHttp || uriResult.Scheme == Uri.UriSchemeHttps);
            if (result)
            {
                newLabel.HoverMouse = MouseCursor.Hand;
                newLabel.Color = Color4.Aqua;
                newLabel.Clicked += NewLabelOnClicked;
            }
            
            newLabel.MouseEnter += NewLabel_MouseEnter;
            newLabel.MouseLeave += NewLabel_MouseLeave;
            Controls.Add(newLabel);
            LastObject = newLabel;
        }

        if (msg.Files.Count > 0)
        {
            int row = 1;
            int filesonrow = 0;
            for (int i = 0; i < msg.Files.Count; i++)
            {
                double lx = (HorPadding * filesonrow) + LastObject.Location.X + (333 * (filesonrow + 1));
                if (lx > Size.X)
                {
                    row++;
                    filesonrow = 0;
                    lx = (HorPadding * filesonrow) + LastObject.Location.X + (333 * (filesonrow + 1));
                }

                filesonrow++;
                IRenderObject cem = await ContentEmbed.GetEmbed(this, msg.Files[i], msg.ChannelID);
                cem.Location = new((int)(lx - 333), (int)(LastObject.Location.Y + 2 + LastObject.Size.Y + (HorPadding * row) + (66 * (row - 1))), 0);
                LastObject = cem;
                Controls.Add(cem);
            }
        }
        if (LastObject is Label ll) Size = new(Size.X, (int)(ll.Location.Y + ll.Size.Y + VerticalPadding));
        else Size = new(Size.X ,(int)(LastObject.Location.Y + LastObject.Size.Y + VerticalPadding));
    }

    private Task NewLabelOnClicked(IRenderObject arg)
    {
        try
        {
            Label m = (arg as Label)!;
            if (OperatingSystem.IsWindows())
                Process.Start(m.Text);
            else if (OperatingSystem.IsLinux())
            {
                if (m.Tag is string s) Process.Start("xdg-open", s);
                else Process.Start("xdg-open", m.Text);
            }
        }
        catch (Exception e)
        {
            Console.WriteLine(e);
        }
        return Task.CompletedTask;
    }

    readonly List<Label> Labels = new();
    private Task NewLabel_MouseLeave(IRenderObject sender)
    {
        if (sender is not Label label) return Task.CompletedTask;
        if (label.Tag is not MainSocketMessage Message) return Task.CompletedTask;
        DateTime time = new DateTime(2022, 1, 1, 0, 0, 0, 0).AddMilliseconds(Message.Id >> 22).ToLocalTime();
        string b = time.ToString("h:mm tt");
        Label[] l = Labels.Where(s => s.Text == b).ToArray();
        if (l.Any())
        {
            Controls.Remove(l.First());
            Labels.Remove(l.First());
        }
        Window!.DrawFrame();
        return Task.CompletedTask;
    }
    
    private Task NewLabel_MouseEnter(IRenderObject sender)
    {
        if (sender is not Label label) return Task.CompletedTask;
        if (label.Tag is not MainSocketMessage Message) return Task.CompletedTask;
        DateTime time = new DateTime(2022, 1, 1, 0, 0, 0, 0).AddMilliseconds(Message.Id >> 22).ToLocalTime();
        Label m = new(Globals.DefaultFont)
        {
            Text = time.ToString("h:mm tt"),
            Location = new((int)(7.5 * Globals.Settings.Scale), label.Location.Y - 13 + (int)label.Font.PixelHeight, 0),
        };
        
        Controls.Add(m);
        Labels.Add(m);
        Window!.DrawFrame();
        return Task.CompletedTask;
    }
}