using System.Reflection;
using GraphicsManager;
using GraphicsManager.Interfaces;
using GraphicsManager.Objects;
using GraphicsManager.Objects.Core;
using Luski.net;
using OpenTK.Graphics.OpenGL4;

namespace Luski.GUI.MainScreen.UI;

public class ServerIcon<TServer> : UserControl where TServer : Server
{
    private Rectangle SelectedRect;
    private static ServerIcon<MainServer>? ser1;
    private static ServerIcon<PublicServer>? ser2;

    public async Task LoadServer()
    {
        if (ser1 is not null)
        {
            ser1.SelectedRect.BackgroundColor = new(26, 26, 26, 255);
            ser1 = null;
        }
        if (ser2 is not null)
        {
            ser2.SelectedRect.BackgroundColor = new(26, 26, 26, 255);
            ser2 = null;
        }

        SelectedRect.BackgroundColor = new(255, 255, 255, 255);
        Globals.UserTextureMap.Clear();
        if (typeof(TServer).FullName == typeof(PublicServer).FullName)
        {
            ser2 = this as ServerIcon<PublicServer>;
            await Globals.ms.LoadPublicServer(ser2!.Server);
        }
        else
        {
            ser1 = this as ServerIcon<MainServer>;
            await Globals.ms.LoadMainServer(ser1!.Server);
        }
    }
    
    public ServerIcon(TServer Server)
    {
        Rectangle r = new(Globals.ms.TextureManager.AddTexture(Tools.GetResourceStream(Assembly.GetExecutingAssembly(),
            "Luski.Resources.Textures.rc.png")))
        {
            Location = new((int)(18 * Globals.Settings.Scale), (int)(8 * Globals.Settings.Scale), 0),
            Size = new((int)(32 * Globals.Settings.Scale)),
            Shader = Rectangle.DefaultAlphaTextureShader[Globals.ms.Context],
            IgnoreHover = true
        };
        Rectangle rr = new(r.Textures[0])
        {
            Location = new((int)(17 * Globals.Settings.Scale), (int)(7 * Globals.Settings.Scale), 0),
            Size = new((int)(34 * Globals.Settings.Scale)),
            Shader = Rectangle.DefaultAlphaShader[Globals.ms.Context],
            BackgroundColor = new(26, 26, 26, 255),
            IgnoreHover = true
        };
        SelectedRect = new(r.Textures[0])
        {
            Location = new((int)(14 * Globals.Settings.Scale), (int)(4 * Globals.Settings.Scale), 0),
            Size = new((int)(40 * Globals.Settings.Scale)),
            Shader = Rectangle.DefaultAlphaShader[Globals.ms.Context],
            BackgroundColor = new(26, 26, 26, 255),
            IgnoreHover = true
        };
        this.Server = Server;
        Texture t = Globals.ms.TextureManager.AddTexture(Server.GetAvatar(CancellationToken.None).Result);
        t.Unit = TextureUnit.Texture1;
        r.Textures.Add(t);
        Controls.Add(SelectedRect);
        Controls.Add(rr);
        Controls.Add(r);
        BackgroundColor = new(26, 26, 26, 255);
        this.Clicked += OnClicked;
        Size = new((int)(68 * Globals.Settings.Scale), (int)(48 * Globals.Settings.Scale));
    }
    
    private Task OnClicked(IRenderObject arg)
    {
        
        return Task.CompletedTask;
    }

    public TServer Server { get; }
}