using System.Reflection;
using GraphicsManager;
using GraphicsManager.Enums;
using GraphicsManager.Interfaces;
using GraphicsManager.Objects;
using GraphicsManager.Objects.Core;
using Luski.net;
using OpenTK.Graphics.OpenGL4;
using OpenTK.Mathematics;
using OpenTK.Windowing.Common;
using OpenTK.Windowing.GraphicsLibraryFramework;

namespace Luski.GUI.StartPage.UI;

public class CreateAccount : UserControl
{
    private RoundedButton button;
    private TexturedTextBox Password, UsernameTextbox, DisplaynameTextBox;
    private Rectangle rec;
    private string? pfp;
    
    public event Func<Task>? ChangeToApp;
    
    public CreateAccount()
    {
        Size = new((int)(312 * Globals.Settings.Scale), (int)(545 * Globals.Settings.Scale));
        Rectangle tt;
        Controls.Add(tt=new Rectangle(Globals.LuskiTexture) { Size = new((int)(179*Globals.Settings.Scale), (int)(189*Globals.Settings.Scale))});
        Label t;
        Controls.Add(t=new Label(Globals.DefaultFont) { Scale = 1.6f * (float)Globals.Settings.Scale, Location = new((int)(199*Globals.Settings.Scale)), Text = "Luski", Color = new(243, 119, 53, 255) });
        t.Location = new((Size.X / 2) - (t.Size.X / 2), t.Location.Y, 0);
        tt.Location = new((Size.X / 2) - (tt.Size.X / 2), tt.Location.Y, 0);
        Controls.Add(UsernameTextbox = new(Globals.LuskiTexture, Globals.DefaultFont, Globals.DefaultFont) { TextLocation = TextLocation.TrueCenterLeft, WatermarkText = "Username", Location = new((int)(27 * Globals.Settings.Scale),(int)(280 * Globals.Settings.Scale), 0), Size = new((int)(261 * Globals.Settings.Scale),(int)(27 * Globals.Settings.Scale)),});
        Controls.Add(Password = new(Globals.LuskiTexture, Globals.DefaultFont, Globals.DefaultFont) { TextLocation = TextLocation.TrueCenterLeft, WatermarkText = "Password", PasswordChar = '●', Location = new(UsernameTextbox.Location.X,(int)(346 * Globals.Settings.Scale), 0), Size = new(UsernameTextbox.Size.X, UsernameTextbox.Location.X), });
        Controls.Add(DisplaynameTextBox = new(Globals.LuskiTexture, Globals.DefaultFont, Globals.DefaultFont) { TextLocation = TextLocation.TrueCenterLeft, WatermarkText = "Displayname", Location = new(UsernameTextbox.Location.X,(int)(400 * Globals.Settings.Scale), 0), Size = new((int)(196 * Globals.Settings.Scale), UsernameTextbox.Location.X), });
        Controls.Add(button = new(Globals.LuskiTexture, Globals.DefaultFont) { Text = "Create Account", Location = new(UsernameTextbox.Location.X, (int)(456 * Globals.Settings.Scale), 0), Size = new(UsernameTextbox.Size.X, (int)(46 * Globals.Settings.Scale)),});
        Controls.Add(rec = new Rectangle(){ Location = new((int)(228 * Globals.Settings.Scale), (int)(384 * Globals.Settings.Scale), 0), Size = new((int)(50 * Globals.Settings.Scale)), BackgroundColor = Color4.Red});
        Password.KeyPress += PasswordOnKeyPress;
        UsernameTextbox.KeyPress += UsernameTextboxOnKeyPress;
        button.Clicked += ButtonOnClicked;
        DisplaynameTextBox.KeyPress += DisplaynameTextBoxOnKeyPress;
        rec.FilesDroped += RecOnFilesDroped;
    }

    private Task DisplaynameTextBoxOnKeyPress(KeyboardKeyEventArgs arg)
    {
        if (arg.Key != Keys.Enter && arg.Key != Keys.KeyPadEnter && arg.Key != Keys.Tab) return Task.CompletedTask;
        if (arg.Key == Keys.Tab && arg.Shift) { Password.Focus(); return Task.CompletedTask; } 
        if (arg.Key == Keys.Enter || arg.Key == Keys.KeyPadEnter) ButtonOnClicked(DisplaynameTextBox);
        return Task.CompletedTask;
    }

    private Task RecOnFilesDroped(string[] arg)
    {
        if (!arg[0].ToLower().EndsWith("png")) return Task.CompletedTask;
        
        pfp = arg[0];
        if (!rec.Textures.Any())
        {
            Controls.Remove(rec);
            Vector3i oldl = rec.Location;
            Vector2i olds = rec.Size;
            rec = new Rectangle(Globals.ms.TextureManager.AddTexture(Tools.GetResourceStream(Assembly.GetExecutingAssembly(),
                "Luski.Resources.Textures.Status.png"))) { Location = oldl, Size = olds };
            Texture tex = Globals.ms.TextureManager.AddTexture(File.OpenRead(arg[0]));
            tex.Unit = TextureUnit.Texture1;
            rec.Shader = Rectangle.DefaultAlphaTextureShader[Globals.ms.Context];
            rec.Textures.Add(tex);
            Controls.Add(rec);
            //rec = new(Globals.ms.TextureManager.AddTexture(File.ReadAllBytes(arg[0]), true)){ Location = new(350, 585), Size = new(100) };Controls.Add(rec);
        }
        else
        {
            rec.Textures[1] = Globals.ms.TextureManager.AddTexture(File.OpenRead(arg[0]));
            rec.Textures[1].Unit = TextureUnit.Texture1;
        }
        
        Window!.DrawFrame();
        return Task.CompletedTask;
    }

    private bool tab;
    private Task UsernameTextboxOnKeyPress(KeyboardKeyEventArgs arg)
    {
        if (arg.Key != Keys.Tab) return Task.CompletedTask;
        if (arg.Key == Keys.Tab){ Password.Focus(); tab = true;}
        return Task.CompletedTask;
    }

    private Task PasswordOnKeyPress(KeyboardKeyEventArgs arg)
    {
        if (tab)
        {
            tab = false;
            return Task.CompletedTask;
        };
        if (arg.Key != Keys.Enter && arg.Key != Keys.KeyPadEnter && arg.Key != Keys.Tab) return Task.CompletedTask;
        if (arg.Key == Keys.Tab && arg.Shift) { UsernameTextbox.Focus(); return Task.CompletedTask; }
        if (arg.Key == Keys.Tab ){ DisplaynameTextBox.Focus(); return Task.CompletedTask; }
        return Task.CompletedTask;
    }

    private Task ButtonOnClicked(IRenderObject arg)
    {
        try
        {
            string[] arr = new string[]
            {
                UsernameTextbox.Text,
                Password.Text,
                DisplaynameTextBox.Text,
                pfp!
            };
            if (arr.Any(s => string.IsNullOrEmpty(s))) return Task.CompletedTask;
            Globals.Luski.MainServer.CreateAccount(UsernameTextbox.Text, Password.Text, DisplaynameTextBox.Text, pfp!, CancellationToken.None);
            ChangeToApp!.Invoke();
        }
        catch (Exception e)
        {
            Console.WriteLine(e);
        }
        return Task.CompletedTask;
    }
}