using System.Reflection;
using System.Text.Json;
using System.Text.Json.Serialization.Metadata;
using GraphicsManager;
using GraphicsManager.Interfaces;
using GraphicsManager.Objects.Core;
using Luski.Clesses;
using Luski.GUI;
using Luski.net;
using Luski.net.Interfaces;
using Luski.net.Structures.Public;
using OpenTK.Graphics.OpenGL4;
using OpenTK.Mathematics;
using OpenTK.Windowing.Common.Input;

namespace Luski;

public static class Globals
{
#pragma warning disable CS8618
    public static Dictionary<Window, List<IRenderObject>> AllowedBehindObjects = new();

    public static void AddBehindObject(this Window w, IRenderObject obj)
    {
        if (!AllowedBehindObjects.ContainsKey(w)) AllowedBehindObjects.Add(w, new());
        if (AllowedBehindObjects[w].Count > 0)
        {
            obj.AllowHoverFromBehind = AllowedBehindObjects[w][0].AllowHoverFromBehind;
        }
        w.Controls.Add(obj);
        AllowedBehindObjects[w].Add(obj);
    }
    
    public static void ToggleBehindObjects(this Window w)
    {
        if (!AllowedBehindObjects.ContainsKey(w)) AllowedBehindObjects.Add(w, new());
        if (AllowedBehindObjects[w].Count > 0)
        {
            bool new_val = !AllowedBehindObjects[w][0].AllowHoverFromBehind;
            foreach (IRenderObject v in AllowedBehindObjects[w])
            {
                v.AllowHoverFromBehind = new_val;
            }
        }
    }
    
    public static void RemoveBehindObject(this Window w, IRenderObject obj, bool purge = true)
    {
        if (!AllowedBehindObjects.ContainsKey(w)) AllowedBehindObjects.Add(w, new());
        AllowedBehindObjects[w].Remove(obj);
        w.Controls.Remove(obj, purge);
        obj.Clean();
        obj = null!;
    }
    
    
    public static bool Download { get; set; } = false;
    public static API Luski { get; } = new();
    public static MainScreenWindow ms;
    private static Texture? gac;

    public static Color4 ToColor4(this Color col)
    {
        return new(col.R, col.G, col.B, col.A);
    }

    public static Texture GetAlphaCircle(this TextureManager tm)
    {
        if (gac is null)
        {
            gac = tm.AddTexture(Tools.GetResourceStream(Assembly.GetExecutingAssembly(),
                "Luski.Resources.Textures.Status.png"));
        }

        return gac;
    }

    public static Dictionary<long, Texture> UserTextureMap = new();
    
    public static async Task<Texture> GetIcon(this SocketUser User)
    {
        if (UserTextureMap.TryGetValue(User.Id, out Texture? t)) return t;
        Stream UserStream = await User.GetAvatar(CancellationToken.None);
        t = Globals.ms.TextureManager.AddTexture(UserStream);
        UserTextureMap.Add(User.Id, t);
        UserStream.Dispose();
        t.Unit = TextureUnit.Texture1;
        return t;
    }
    
    public static async Task<Texture> GetIcon(this IUser User)
    {
        if (UserTextureMap.TryGetValue(User.Id, out Texture? t)) return t;
        Stream UserStream = await User.GetAvatar(CancellationToken.None);
        t = Globals.ms.TextureManager.AddTexture(UserStream);
        UserTextureMap.Add(User.Id, t);
        UserStream.Dispose();
        t.Unit = TextureUnit.Texture1;
        return t;
    }

    public static Settings Settings { get; set; }

    public static FontFamily DefaultFontFamly { get; set; }
    public static FontInteraction DefaultFont { get; set; }
    public static FontInteraction TopTimeFont { get; set; }
    public static FontInteraction MessageFont { get; set; }
    public static FontInteraction SmallTimeFont { get; set; }

    public static PublicServer? GetCurentPublicServer()
    {
        if (Luski.LoadedServers.Count > 0)
            return Luski.LoadedServers[0];
        else
            return null;
    }
    
    public static UpdaterSettings UpdaterSettings { get; set; }

    public static Texture LuskiTexture;

    public static TResult GetSettings<TResult>(string path, JsonTypeInfo<TResult> TypeInfo) where TResult : new()
    {
        TResult? @out;
        if (!File.Exists(path))
        {
            @out = new();
            File.WriteAllText(path, JsonSerializer.Serialize(@out, TypeInfo));
        }

        try
        {
            @out = JsonSerializer.Deserialize(File.ReadAllText(path), TypeInfo);
            if (@out is null)
            {
                @out = new();
            }
        }
        catch
        {
            @out = new();
        }
        File.WriteAllText(path, JsonSerializer.Serialize(@out, TypeInfo));
        return @out;
    }
    
    public static string JT
    {
        get
        {
            string tmp = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "JacobTech");
            if (OperatingSystem.IsLinux())
            {
                tmp = Path.Combine(Environment.GetEnvironmentVariable("HOME")!, ".config/");
                tmp += "JacobTech";
            }

            if (!Directory.Exists(tmp)) Directory.CreateDirectory(tmp);
            if (!Directory.Exists(Path.Combine(tmp, "Luski"))) Directory.CreateDirectory(Path.Combine(tmp, "Luski"));
            return tmp;
        }
    }
    public static WindowIcon Icon { get; set; }
    
    public static string LuskiPath
    {
        get
        {
            return Path.Combine(JT, "Luski");
        }
    }

    public static bool Empty(string? str)
    {
        return (string.IsNullOrEmpty(str) || string.IsNullOrWhiteSpace(str));
    }
}
#pragma warning restore CS8618